#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const kleur = __importStar(require("kleur/colors"));
const log_update_1 = __importDefault(require("log-update"));
const path = __importStar(require("path"));
const sade_1 = __importDefault(require("sade"));
const scan_task_1 = require("./scan-task");
const utilities_1 = require("./utilities");
const aggregate_1 = require("./aggregate");
/*
This is a require because if it was an import, TS would copy package.json to `dist`
If TS copied package.json to `dist`, npm would not publish the JS files in `dist`
Since it is a require, TS leaves it as-is, which means that the require path
has to be relative to the built version of this file in the dist folder
It may in the future make sense to use a bundler to combine all the dist/ files into one file,
(including package.json) which would eliminate this problem
*/
// eslint-disable-next-line @cloudfour/typescript-eslint/no-var-requires
const { version } = require('../package.json');
const symbols = {
    error: kleur.red('✖'),
    success: kleur.green('✔'),
};
sade_1.default('lighthouse-parade <url> [dataDirectory]', true)
    .version(version)
    .describe('Crawls the site at the provided URL, recording the lighthouse scores for each URL found. The lighthouse data will be stored in the provided directory, which defaults to ./data/YYYY-MM-DDTTZ_HH_MM')
    .option('--ignore-robots', "Crawl pages even if they are listed in the site's robots.txt", false)
    .option('--crawler-user-agent', 'Pass a user agent string to be used by the crawler (not by Lighthouse)')
    .option('--lighthouse-concurrency', 'Control the maximum number of ligthhouse reports to run concurrently', os.cpus().length - 1)
    .action((url, 
// eslint-disable-next-line default-param-last
dataDirPath = path.join(process.cwd(), 'lighthouse-parade-data', utilities_1.usefulDirName()), opts) => {
    // We are attempting to parse the URL here, so that if the user passes an invalid URL,
    // the prorgam will exit here instead of continuing (which would lead to a more confusing error)
    // eslint-disable-next-line no-new
    new URL(url);
    const ignoreRobotsTxt = opts['ignore-robots'];
    const reportsDirPath = path.join(dataDirPath, 'reports');
    fs.mkdirSync(reportsDirPath, { recursive: true });
    const userAgent = opts['crawler-user-agent'];
    if (userAgent !== undefined && typeof userAgent !== 'string') {
        throw new Error('--crawler-user-agent flag must be a string');
    }
    const lighthouseConcurrency = opts['lighthouse-concurrency'];
    const scanner = scan_task_1.scan(url, {
        ignoreRobotsTxt,
        dataDirectory: dataDirPath,
        lighthouseConcurrency,
    });
    const urlStates = new Map();
    const frames = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏'];
    let i = 0;
    const printLine = (url, state, error) => {
        const frame = kleur.blue(frames[i]);
        const statusIcon = error
            ? symbols.error
            : state === 0 /* Pending */
                ? ' '
                : state === 1 /* ReportInProgress */
                    ? frame
                    : symbols.success;
        let output = `${statusIcon} ${url}`;
        if (error) {
            output += `\n  ${kleur.gray(error.toString())}`;
        }
        return output;
    };
    const render = () => {
        const pendingUrls = [];
        const currentUrls = [];
        urlStates.forEach(({ state, error }, url) => {
            if (state === 2 /* ReportComplete */)
                return;
            const line = `${printLine(url, state, error)}\n`;
            if (state === 0 /* Pending */)
                pendingUrls.push(line);
            else
                currentUrls.push(line);
        });
        const numPendingToDisplay = Math.min(Math.max(process.stdout.rows - currentUrls.length - 3, 1), pendingUrls.length);
        const numHiddenUrls = numPendingToDisplay === pendingUrls.length
            ? ''
            : kleur.dim(`\n...And ${pendingUrls.length - numPendingToDisplay} more pending`);
        log_update_1.default(currentUrls.join('') +
            pendingUrls.slice(0, numPendingToDisplay).join('') +
            numHiddenUrls);
    };
    const intervalId = setInterval(() => {
        i = (i + 1) % frames.length;
        render();
    }, 80);
    /**
     * Allows you to run a console.log that will output _above_ the persistent logUpdate log
     * Pass a callback where you run your console.log or console.error
     */
    const printAboveLogUpdate = (cb) => {
        log_update_1.default.clear();
        cb();
        render();
    };
    const log = (...messages) => printAboveLogUpdate(() => console.log(...messages));
    const urlsFile = path.join(dataDirPath, 'urls.csv');
    fs.writeFileSync(urlsFile, 'URL,content_type,bytes,response\n');
    const urlsStream = fs.createWriteStream(urlsFile, { flags: 'a' });
    scanner.on('urlFound', (url, contentType, bytes, statusCode) => {
        urlStates.set(url, { state: 0 /* Pending */ });
        const csvLine = [
            JSON.stringify(url),
            contentType,
            bytes,
            statusCode,
        ].join(',');
        urlsStream.write(`${csvLine}\n`);
    });
    scanner.on('reportBegin', (url) => {
        urlStates.set(url, { state: 1 /* ReportInProgress */ });
    });
    scanner.on('reportFail', (url, error) => {
        urlStates.set(url, { state: 2 /* ReportComplete */, error });
        log(printLine(url, 2 /* ReportComplete */, error));
    });
    scanner.on('reportComplete', (url, reportData) => {
        urlStates.set(url, { state: 2 /* ReportComplete */ });
        log(printLine(url, 2 /* ReportComplete */));
        const reportFileName = utilities_1.makeFileNameFromUrl(url, 'csv');
        fs.writeFileSync(path.join(reportsDirPath, reportFileName), reportData);
    });
    scanner.on('info', (message) => {
        log(message);
    });
    scanner.promise.then(async () => {
        clearInterval(intervalId);
        console.log('Aggregating reports...');
        await aggregate_1.aggregateCSVReports(dataDirPath);
        console.log('DONE!');
    });
})
    .parse(process.argv);
